@@ -1,5 +1,6 @@
 /*
  *  $Id: platform.h,v 1.2 2009-11-24 23:53:40 sampo Exp $
+ * http://support.microsoft.com/kb/190351
  */
 
 #ifndef _platform_h
@@ -9,26 +10,60 @@
 
 #ifdef MINGW
 
+#include <winsock2.h>
 #include <windows.h>
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
+#define MS_LONG LONG
+#define MKDIR(d,p) _mkdir(d)
+#define GETTIMEOFDAY(tv, tz) ((tv) ? (((tv)->tv_sec = time(0)) && ((tv)->tv_usec = 0)) : -1)
+#define GMTIME_R(secs,stm) do { struct tm* stx_tm = gmtime(&(secs)); if (stx_tm) memcpy(&stm, stx_tm, sizeof(struct tm)); } while(0)   /* *** still not thread safe */
+//#define GMTIME_R(t, res) gmtime_r(&(t),&(res))
+
+#define MINGW_RW_PERM (GENERIC_READ | GENERIC_WRITE)
+
+#define fdstdin  (GetStdHandle(STD_INPUT_HANDLE))
 #define fdstdout (GetStdHandle(STD_OUTPUT_HANDLE))
 /*#define fdtype HANDLE   see zx.h */
 #define BADFD (INVALID_HANDLE_VALUE)
 #define closefile(x) (CloseHandle(x)?0:-1)
 #define openfile_ro(path) zx_CreateFile((path), GENERIC_READ, FILE_SHARE_READ, 0 /*security*/, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0)
+
+HANDLE zx_CreateFile(LPCTSTR lpFileName, 
+		     DWORD dwDesiredAccess, DWORD dwShareMode, 
+		     LPSECURITY_ATTRIBUTES lpSecurityAttributes, DWORD dwCreationDisposition, 
+		     DWORD dwFlagsAndAttributes, HANDLE hTemplateFile);
+
+  //#define read(fd, buf, siz) _read((fd),(buf),(siz))
+  //#define write(fd, buf, cnt) _write((fd),(buf),(cnt))
+#define pipe(fds) _pipe(fds, 64*1024, _O_BINARY)
+/*#define MKDIR(P,M)  _mkdir(P)*/
+#define uid_t int
+#define gid_t int
 #define geteuid() 0
 #define getegid() 0
 #define stat(X,Y) zx_stat(X,Y)
+#define openlog(a,b,c)
+#define syslog(a,...)
+#define closelog()
+#define fcntl(fd,cmd,...) (-1)  /* always fail: mingw does not have fcntl(2) */
+#define nice(x) 0
 
+#define F_GETFL 3
+#define F_SETFL 4
+#define F_SETFD 2
 #define LOCK_EX 2
 #define LOCK_UN 8
-#define O_SYNC 04010000
+#define O_SYNC     04010000
+#define O_NONBLOCK 04000
+#define O_NDELAY   O_NONBLOCK
+#define WNOHANG 1
 
 #ifdef WIN32CL
+//#define intptr_t INT_PTR
 /* The directory handling is quite different on Windows. The following
  * posix wrappers are implemented in zxdirent.c */
 typedef struct DIR DIR;
@@ -59,24 +94,18 @@ typedef struct stack_st STACK;  /* MSVC seems to have some problem with openssl/
 #define getcwd(b,n) "getcwd not supported on Win32"  /* *** consider GetCurrentDirectory() */
 unsigned int sleep(unsigned int secs);
 unsigned int alarm(unsigned int secs);
-
-HANDLE zx_CreateFile(LPCTSTR lpFileName, 
-		     DWORD dwDesiredAccess, DWORD dwShareMode, 
-		     LPSECURITY_ATTRIBUTES lpSecurityAttributes, DWORD dwCreationDisposition, 
-		     DWORD dwFlagsAndAttributes, HANDLE hTemplateFile);
-
 #else
 #include <dirent.h>
-#endif
+#endif /* WIN32CL */
 
 /* Windows thread identification is a mess:
  * - thread ID returned by GetCurrentThreadId() is like POSIX thread ID except
- *   that almost none of the windows thread API functions accept it as argument.
+ *   that almost none of the windows thread API functions accept it as an argument.
  *   They need a handle instead.
  * - Handle can not be obtained easily. Instead GetCurrentThread() returns a
  *   pseudohandle (-1) that has only limited usability.
  * - It is not clear what _beginthread() returns. Presumably a handle.
- * In the end we adopt keeping around the thread ID and using OpenThread(0,0,tid)
+ * In the end, we adopt keeping around the thread ID and using OpenThread(0,0,tid)
  * to resolve it to a real thread handle when needed. */
 #define pthread_t DWORD    /* what pthread_self() returns, or GetCurrentThreadId() */
 #define pthread_self() GetCurrentThreadId()  /* Returns an ID, not a handle */
@@ -97,7 +126,8 @@ HANDLE zx_CreateFile(LPCTSTR lpFileName,
 
 #else
 
-/* NOT MINGW nor WIN32CL (i.e. its Unix) */
+/* ============================================================================
+ * NOT MINGW nor WIN32CL (i.e. its Unix) */
 
 #include <dirent.h>
 
@@ -105,10 +135,17 @@ HANDLE zx_CreateFile(LPCTSTR lpFileName,
 extern "C" {
 #endif
 
-#define fdstdout 1
+#define MKDIR(d,p) mkdir((d),(p))
+#define GETTIMEOFDAY gettimeofday
+#define GMTIME_R(t, res) gmtime_r(&(t),&(res))
+
+#define fdstdin  0  /* fileno(stdin) */
+#define fdstdout 1  /* fileno(stdout) */
 /*#define fdtype int   see zx.h */
 #define BADFD (-1)
+#define SOCKET fdtype
 #define closefile(x) close(x)
+#define closesocket(x) close(x)
 #define openfile_ro(path) open((path),O_RDONLY)
 
 #if !defined(_UNISTD_H) && !defined(_UNISTD_H_)
@@ -126,6 +163,7 @@ int dup2(int,int);
 int execl(const char *path, const char *arg, ...);
 int fcntl(int fd, int cmd, ...);         /* Preferred */
 int fork(void);
+int execve(const char* f, char *const argv[], char *const envp[]);
 char* getcwd(char* buf, size_t size);
 int geteuid(void);
 int getegid(void);
@@ -136,13 +174,20 @@ int lseek(int fd, int offset, int whence);
 int pipe(int fd[2]);
 int read(int fd, void* buf, int count);
 int rmdir(const char *pathname);
+int getuid();
+int getgid();
 int setuid(int);
 int setgid(int);
-int symlink(const char *oldpath, const char *newpath);
-int unlink(const char *pathname);
+int setsid();
+int symlink(const char* oldpath, const char* newpath);
+int unlink(const char* pathname);
 int write(int fd, void* buf, int count);
 unsigned int sleep(unsigned int secs);
 unsigned int alarm(unsigned int secs);
+int fchown(int fd, uid_t owner, gid_t group);
+int gethostname(char* name, size_t len);
+int chroot(const char* path);
+int nice(int inc);
 #define F_LOCK 1
 #define F_ULOCK 0
 #endif
