@@ -1,5 +1,5 @@
 /* zxidcurl.c  -  libcurl interface for making SOAP calls and getting metadata
- * Copyright (c) 2013 Synergetics NV (sampo@synergetics.be), All Rights Reserved.
+ * Copyright (c) 2013-2014 Synergetics NV (sampo@synergetics.be), All Rights Reserved.
  * Copyright (c) 2010-2011 Sampo Kellomaki (sampo@iki.fi), All Rights Reserved.
  * Copyright (c) 2006-2008 Symlabs (symlabs@symlabs.com), All Rights Reserved.
  * Author: Sampo Kellomaki (sampo@iki.fi)
@@ -15,6 +15,8 @@
  * 1.2.2010,   removed arbitrary limit on SOAP response size --Sampo
  * 11.12.2011, refactored HTTP GET client --Sampo
  * 26.10.2013, improved error reporting on credential expired case --Sampo
+ * 12.3.2014,  added partial mime multipart support --Sampo
+ * 27.5.2014,  Added feature to stop parsing after end of first top level tag has been seen --Sampo
  *
  * See also: http://hoohoo.ncsa.uiuc.edu/cgi/interface.html (CGI specification)
  *           http://curl.haxx.se/libcurl/
@@ -173,7 +175,7 @@ char* zxid_http_get(zxid_conf* cf, const char* url, char** lim)
  * returns:: HTTP body of the response */
 
 /* Called by:  zxid_soap_call_raw */
-struct zx_str* zxid_http_post_raw(zxid_conf* cf, int url_len, const char* url, int len, const char* data)
+struct zx_str* zxid_http_post_raw(zxid_conf* cf, int url_len, const char* url, int len, const char* data, const char* SOAPaction)
 {
 #ifdef USE_CURL
   struct zx_str* ret;
@@ -181,7 +183,7 @@ struct zx_str* zxid_http_post_raw(zxid_conf* cf, int url_len, const char* url, i
   struct zxid_curl_ctx rc;
   struct zxid_curl_ctx wc;
   struct curl_slist content_type;
-  struct curl_slist SOAPaction;
+  struct curl_slist SOAPaction_curl;
   char* urli;
   rc.buf = rc.p = ZX_ALLOC(cf->ctx, ZXID_INIT_SOAP_BUF+1);
   rc.lim = rc.buf + ZXID_INIT_SOAP_BUF;
@@ -222,18 +224,15 @@ struct zx_str* zxid_http_post_raw(zxid_conf* cf, int url_len, const char* url, i
   curl_easy_setopt(cf->curl, CURLOPT_READFUNCTION, zxid_curl_read_data);
 
   ZERO(&content_type, sizeof(content_type));
-  content_type.data = "Content-Type: text/xml";
-  ZERO(&SOAPaction, sizeof(SOAPaction));
-#if 1
-  SOAPaction.data = "SOAPAction: \"\"";  /* Empty SOAPAction is the ID-WSF (and SAML?) standard */
-#else
-  /* Evil stuff: some implementations, especially Apache AXIS, are very
-   * picky about SOAPAction header. */
-  //SOAPaction.data = "SOAPAction: \"http://ws.apache.org/axis2/TestPolicyPortType/authRequestRequest\"";
-  SOAPaction.data = "SOAPAction: \"authRequest\"";
-#endif
-  SOAPaction.next = &content_type;    //curl_slist_append(3)
-  curl_easy_setopt(cf->curl, CURLOPT_HTTPHEADER, &SOAPaction);
+  content_type.data = cf->wsc_soap_content_type; /* SOAP11: "Content-Type: text/xml" */
+  if (SOAPaction) {
+    ZERO(&SOAPaction_curl, sizeof(SOAPaction_curl));
+    SOAPaction_curl.data = (char*)SOAPaction;
+    SOAPaction_curl.next = &content_type;    //curl_slist_append(3)
+    curl_easy_setopt(cf->curl, CURLOPT_HTTPHEADER, &SOAPaction_curl);
+  } else {
+    curl_easy_setopt(cf->curl, CURLOPT_HTTPHEADER, &content_type);
+  }
   
   INFO("----------- call(%s) -----------", urli);
   DD("SOAP_CALL post(%.*s) len=%d\n", len, data, len);
@@ -261,13 +260,16 @@ struct zx_str* zxid_http_post_raw(zxid_conf* cf, int url_len, const char* url, i
     ERR("Failed post to url(%s) CURLcode(%d) CURLerr(%s)", urli, res, CURL_EASY_STRERR(res));
     DD("buf(%.*s)", rc.lim-rc.buf, rc.buf);
   }
+
+  /*curl_easy_getinfo(cf->curl, CURLINFO_CONTENT_TYPE, char*);*/
+
   UNLOCK(cf->curl_mx, "curl-soap");
   ZX_FREE(cf->ctx, urli);
   rc.lim = rc.p;
   rc.p[0] = 0;
 
   DD("SOAP_CALL got(%s)", rc.buf);
-  D_XML_BLOB(cf, "SOAPCALL GOT", -2, rc.buf);
+  D_XML_BLOB(cf, "SOAPCALL GOT", rc.lim - rc.buf, rc.buf);
   
   ret = zx_ref_len_str(cf->ctx, rc.lim - rc.buf, rc.buf);
   return ret;
@@ -328,6 +330,54 @@ zxid_entity* zxid_get_meta_ss(zxid_conf* cf, struct zx_str* url)
   return zxid_get_meta(cf, zx_str_to_c(cf->ctx, url));
 }
 
+/*() Locate first SOAP Envelope using simple heuristic
+ * searching for string "Envelope" (and related namespace).
+ * Typically this allows extraction of SOAP envelope from deep
+ * inside MIME multipart message (MTOM+xop aka SOAP with attachments) */
+
+const char* zxid_locate_soap_Envelope(const char* haystack)
+{
+  const char* q;
+  const char* p = strstr(haystack, zx_xmlns_e);
+  if (p) {
+    for (q = p-1; q >= haystack; --q)
+      if (*q == '<') break;
+    if (q < haystack)
+      return 0;
+    p = zx_memmem(q, p-q, "Envelope", sizeof("Envelope")-1);
+    if (p)
+      return q;
+  } else {
+    D("Trying to detect namespaceless Envelope %d",0);
+    p = strstr(haystack, "Envelope");
+    if (p && p > haystack) {
+      --p;
+      switch (*p) {
+      case '<': return p;
+      case ':': /* Scan over namespace prefix */
+	for (--p; p > haystack; --p)
+	  if (!AZaz_09_dash(*p)) break;
+	if (*p == '<')
+	  return p;
+	/* else fall through to error */
+      default:
+	return 0;
+      }
+    }
+  }
+  return 0;
+}
+
+/*() Return Content-Type header from last HTTP response.
+ * This could be used to detect MIME multipart boundary, for example. */
+
+const char* zxid_get_last_content_type(zxid_conf* cf)
+{
+  char* ct;
+  curl_easy_getinfo(cf->curl, CURLINFO_CONTENT_TYPE, &ct);
+  return ct;
+}
+
 /* ============== SOAP Call ============= */
 
 /*(i) Send SOAP request and wait for response. Send the message to the
@@ -342,7 +392,7 @@ zxid_entity* zxid_get_meta_ss(zxid_conf* cf, struct zx_str* url)
  *
  * The underlying HTTP client is libcurl. While libcurl is documented to
  * be "entirely thread safe", one limitation is that curl handle can not
- * be shared between threads. Since we keep the curl handle a part
+ * be shared between threads. Since we keep the curl handle as a part
  * of the configuration object, which may be shared between threads,
  * we need to take a lock for duration of the curl operation. Thus any
  * given configuration object can have only one HTTP request active
@@ -357,17 +407,48 @@ struct zx_root_s* zxid_soap_call_raw(zxid_conf* cf, struct zx_str* url, struct z
   struct zx_root_s* r;
   struct zx_str* ret;
   struct zx_str* ss;
+  char soap_action_buf[1024];
+  char* soap_act;
+  const char* env_start;
 
   ss = zx_easy_enc_elem_opt(cf, &env->gg);
   DD("ss(%.*s) len=%d", ss->len, ss->s, ss->len);
-  ret = zxid_http_post_raw(cf, url->len, url->s, ss->len, ss->s);
+
+  if (cf->soap_action_hdr && strcmp(cf->soap_action_hdr,"#inhibit")) {
+    if (!strcmp(cf->soap_action_hdr,"#same")) {
+      if (env->Header && env->Header->Action && ZX_GET_CONTENT_S(env->Header->Action)) {
+	snprintf(soap_action_buf,sizeof(soap_action_buf), "SOAPAction: \"%.*s\"", ZX_GET_CONTENT_LEN(env->Header->Action), ZX_GET_CONTENT_S(env->Header->Action));
+	soap_action_buf[sizeof(soap_action_buf)-1] = 0;
+	soap_act = soap_action_buf;
+	D("SOAPaction(%s)", soap_action_buf);
+      } else {
+	ERR("e:Envelope/e:Headers/a:Action SOAP header is malformed %p", env->Header);
+      }
+    } else {
+      snprintf(soap_action_buf,sizeof(soap_action_buf), "SOAPAction: \"%s\"", cf->soap_action_hdr);
+      soap_action_buf[sizeof(soap_action_buf)-1] = 0;
+      soap_act = soap_action_buf;
+    }
+  } else
+    soap_act = 0;
+  
+  ret = zxid_http_post_raw(cf, url->len, url->s, ss->len, ss->s, soap_act);
   zx_str_free(cf->ctx, ss);
   if (ret_enve)
     *ret_enve = ret?ret->s:0;
   if (!ret)
     return 0;
+  
+  env_start = zxid_locate_soap_Envelope(ret->s);
+  if (!env_start) {
+    ERR("SOAP response does not have Envelope element url(%.*s)", url->len, url->s);
+    D_XML_BLOB(cf, "NO ENVELOPE SOAP RESPONSE", ret->len, ret->s);
+    ZX_FREE(cf->ctx, ret);
+    return 0;
+  }
 
-  r = zx_dec_zx_root(cf->ctx, ret->len, ret->s, "soap_call");
+  cf->ctx->top1 = 1;  /* Stop parsing after first toplevel <e:Envelope> */
+  r = zx_dec_zx_root(cf->ctx, ret->len - (env_start - ret->s), env_start, "soap_call");
   if (!r || !r->Envelope || !r->Envelope->Body) {
     ERR("Failed to parse SOAP response url(%.*s)", url->len, url->s);
     D_XML_BLOB(cf, "BAD SOAP RESPONSE", ret->len, ret->s);
@@ -450,7 +531,7 @@ int zxid_soap_cgi_resp_body(zxid_conf* cf, zxid_ses* ses, struct zx_e_Body_s* bo
     ZX_ADD_KID(env->Body, Fault, ses->curflt);
   }
   
-  zxid_wsf_decor(cf, ses, env, 1);
+  zxid_wsf_decor(cf, ses, env, 1, 0);
   ss = zx_easy_enc_elem_opt(cf, &env->gg);
 
   if (cf->log_issue_msg) {
