@@ -206,13 +206,13 @@ static void zxsig_canon_crnl_inplace(struct zx_str* ss)
  * and metadata of the signing party. Trust in the certificate must have
  * been established by other means.
  *
- * c::      ZX context. Used for memory allocation.
- * cert::   Signing party's certificate (public key), typically from metadata. If NULL,
- *          then only the hashes (and hence canonicalization) are checked, but the
- *          public key crypto part is not performed, and ZXSIG_BAD_CERT is returned.
- * sig::    Parsed XML-DSIG data structure
- * n::      Number of elements in the sref array
- * sref::   An array of <reference sref, xml data structure blob> tuples that are
+ * c::    ZX context. Used for memory allocation.
+ * cert:: Signing party's certificate (public key), typically from metadata. If NULL,
+ *     then only the hashes (and hence canonicalization) are checked, but the
+ *     public key crypto part is not performed, and ZXSIG_BAD_CERT is returned.
+ * sig::  Parsed XML-DSIG data structure
+ * n::    Number of elements in the sref array
+ * sref:: An array of <reference sref, xml data structure blob> tuples that are
  *     referenced by the signature
  * return:: ZXSIG value. 0 (ZXSIG_OK) means success. Any other value is some sort of failure */
 
@@ -558,8 +558,8 @@ struct zx_str* zxenc_symkey_dec(zxid_conf* cf, struct zx_xenc_EncryptedData_s* e
     return 0;
   }
   
-  if (!symkey) {
-    ERR("Symmetric key missing. Perhaps public key operation to recover symmetric key failed (e.g. missing private key, or private key does not match public key). Perhaps the programmer simply failed to pass correct arguments to this function. %d", 0);
+  if (!symkey || !symkey->s) {
+    ERR("Symmetric key missing. Perhaps public key operation to recover symmetric key failed (e.g. missing private key, or private key does not match public key). Perhaps the programmer simply failed to pass correct arguments to this function. %p", symkey);
     zxlog(cf, 0, 0, 0, 0, 0, 0, 0, "N", "C", "EMISS", 0, "no symkey");
     return 0;
   }
@@ -567,7 +567,12 @@ struct zx_str* zxenc_symkey_dec(zxid_conf* cf, struct zx_xenc_EncryptedData_s* e
   raw.s = ZX_ALLOC(cf->ctx, SIMPLE_BASE64_PESSIMISTIC_DECODE_LEN(ss->len));
   lim = unbase64_raw(ss->s, ss->s+ss->len, raw.s, zx_std_index_64);
   raw.len = lim - raw.s;
-  
+
+  if (!ed->EncryptionMethod || !ed->EncryptionMethod->Algorithm) {
+    ERR("EncryptionMethod or Algorithm element not found or malformed %p", ed->EncryptionMethod);
+    zxlog(cf, 0, 0, 0, 0, 0, 0, 0, "N", "C", "EMISS", 0, "no or bad EncryptionMethod");
+    return 0;
+  }
   ss = &ed->EncryptionMethod->Algorithm->g;
   if (sizeof(ENC_ALGO_TRIPLEDES_CBC)-1 == ss->len
       && !memcmp(ENC_ALGO_TRIPLEDES_CBC, ss->s, sizeof(ENC_ALGO_TRIPLEDES_CBC)-1)) {
@@ -593,6 +598,11 @@ struct zx_str* zxenc_symkey_dec(zxid_conf* cf, struct zx_xenc_EncryptedData_s* e
     zxlog(cf, 0, 0, 0, 0, 0, 0, 0, "N", "C", "ECRYPT", 0, "unsupported key transformation method");
     return 0;
   }
+  if (!ss) {
+    ERR("Symmetric key decryption failure. Perhaps symmetric key derived from assymmetric level is wrong, i.e. certificate used for encryption does not match the private key %d", 0);
+    zxlog(cf, 0, 0, 0, 0, 0, 0, 0, "N", "C", "ECRYPT", 0, "symmetric decrypt failed");
+    return 0;
+  }
   ZX_FREE(cf->ctx, raw.s);
   DD("plain(%.*s)", ss->len, ss->s);
   D_XML_BLOB(cf, "PLAIN", ss->len, ss->s);
@@ -741,6 +751,10 @@ struct zx_xenc_EncryptedData_s* zxenc_symkey_enc(zxid_conf* cf, struct zx_str* d
   DD("Plaintext(%.*s)", data->len, data->s);
   D_XML_BLOB(cf, "PLAINTEXT", data->len, data->s);
   ss = zx_raw_cipher(cf->ctx, "AES-128-CBC", 1, symkey, data->len, data->s, 16, 0);
+  if (!ss) {
+    ERR("Symmetric encryption failed %d",0);
+    return 0;
+  }
   b64 = zx_new_len_str(cf->ctx, SIMPLE_BASE64_LEN(ss->len));
   base64_fancy_raw(ss->s, ss->len, b64->s, std_basis_64, 0, 0, 0, '=');
   zx_str_free(cf->ctx, ss);
