@@ -15,6 +15,7 @@
  * 20.6.2011, improved error reporting to show cwd in vopen_fd_from_path() --Sampo
  * 17.8.2012, added socket specific utilities --Sampo
  * 21.6.2013, added wild card matcher --Sampo
+ * 30.11.2013, fixed read 1 past end in base64_fancy_raw() found by valgrind --Sampo
  */
 
 #include "platform.h"
@@ -56,7 +57,7 @@ struct flock errmac_unlk = { F_UNLCK, SEEK_SET, 0, 1 };
 
 int close_file(fdtype fd, const char* logkey);
 
-/*() Report brokenness of snprintf() */
+/*(-) Report brokenness of snprintf() */
 
 /* Called by:  vname_from_path, write_all_fd_fmt, write_all_path_fmt, zx_alloc_vasprintf, zxbus_log_receipt, zxid_epr_path */
 void platform_broken_snprintf(int n, const char* where, int maxlen, const char* fmt)
@@ -146,7 +147,7 @@ fdtype open_fd_from_path(int flags, int mode, const char* logkey, int reperr, co
  * want is satisfied or error happens. May block (though usually will not if
  * the file is in cache or local disk) in process. Buffer p must have been allocated.
  * Return value reflects last got, i.e. what last read(2) system call returned.
- * got_all reflects the total number of bytes received. */
+ * got_all reflects the total number of bytes received. Does not nul terninate. */
 
 /* Called by: */
 int read_all_fd(fdtype fd, char* p, int want, int* got_all)
@@ -446,7 +447,7 @@ int write_all_path_fmt(const char* logkey, int maxlen, char* buf, const char* pa
  * (flushing the data).  Will perform file locking to ensure
  * consistent results. Will create the file if needed, but will not
  * create parent directories. Up to two items of data can
- * be written/appended. If you have only one item, supply null
+ * be written or appended. If you have only one item, supply null
  * for the second. For overwrite behaviour supply seeky=SEEK_SET and
  * flag=O_TRUNC (the seek offset is always 0). For append behaviour
  * supply seeky=SEEK_END and flag=O_APPEND.
@@ -586,7 +587,7 @@ linkrest:
   fd_to = open(to, O_RDWR | O_CREAT, 0666);
 #endif
   if (fd_to == BADFD) {
-      perror("openfile_ro");
+      perror("openfile_rw");
       ERR("%s: Error opening to(%s) euid=%d egid=%d", logkey, to, geteuid(), getegid());
       return -1;
   }
@@ -749,10 +750,10 @@ char* base64_fancy_raw(const char* p, int len, /* input and its length */
   
   /* Post processing to handle the last line, which is often incomplete. */
   
-  c1 = *p++;
   switch (len) {
   case 2:
-    c2 = *p++;  // *** len==1 causes bug if no null term
+    c1 = *p++;
+    c2 = *p;
     *r++ = basis_64[c1>>2];
     *r++ = basis_64[((c1 & 0x0003)<< 4) | ((c2 & 0x00f0) >> 4)];
     *r++ = basis_64[(c2 & 0x000f) << 2];
@@ -760,6 +761,7 @@ char* base64_fancy_raw(const char* p, int len, /* input and its length */
       *r++ = eq_pad;
     break;
   case 1:
+    c1 = *p;
     *r++ = basis_64[c1>>2];
     *r++ = basis_64[(c1 & 0x0003)<< 4];
     if (eq_pad) {
@@ -1180,6 +1182,9 @@ scan_end:
     goto again;
   }
 
+  if (*qs == '[')                                /* Section header line, treat like comment */
+    goto scan_end;
+
   for (; *qs == '&'; ++qs) ;                     /* Skip over & or && */
   if (!*qs)
     return 0;
